==> em_tasks-0.1.35-py2.py3-none-any.whl: em_tasks/utils/amz_uploadable_products_filter.py <==
# -*- coding: utf-8 -*-

from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.store.store_util import StoreUtil
from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.store.models import *


class AmzUploadableProductsFilter():
  def __init__(self, marketplace, product_service, offer_service, price_caculator, output_path, location_id=None):
    self.product_service = product_service
    self.offer_service = offer_service
    self.price_calc = price_caculator
    self.location_id = location_id
    self.marketplace = marketplace.lower()
    self.pc = AmzProductConverter(self.marketplace)
    self.indice_name = 'amz_products_api_{}'.format(marketplace.lower())
    self.condition = 'new'
    self.output_path = os.path.abspath(os.path.expanduser(output_path))
    output_dir = os.path.dirname(self.output_path)
    if not os.path.isdir(output_dir):
      os.makedirs(output_dir)
    self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')

  def filter(self):
    try:
      blacklisted_path = './amz_{}_blacklisted.txt'.format(self.marketplace)
      records = dict()
      for keyword in BlacklistKeyword.select().order_by(BlacklistKeyword.id.desc()).dicts():
        k = keyword['keywords']
        records[k]= []

        query = {
          'query': {
            'query': {
              'multi_match': {
                'query': k,
                'type': 'phrase',
                'fields': ['brand^10', 'title']
              }
            }
          }
        }
        for prod_id, _ in self.product_service.load_products(self.indice_name, query):
          records[prod_id] = None

        with open(blacklisted_path, 'w+', encoding='utf-8', errors='ignore') as fh:
          fh.write("\n".join(list(records.keys())))

      logger.debug('[BlacklistedProductsLoaded] ProductsCnt: %s', len(records))

      uploaded_prods = dict()
      for prod in StoreUtil.get_amz_products(self.marketplace):
        uploaded_prods[prod['asin']] = None

      logger.debug('[UploadedProductsLoaded] ProductsCnt: %s', len(uploaded_prods))

      prods_buf = dict()
      batch_size = 500
      for prod_id, prod in self.product_service.load_products(self.indice_name):
        exist = prod.get('existence', True)
        if not exist:
          continue

        has_only_default_variant = prod.get('has_only_default_variant', True)
        if not has_only_default_variant:
          continue

        if prod_id in records or prod_id in uploaded_prods:
          continue

        weight = self.pc.get_weight(prod)
        if weight and weight > 1.5:
          continue

        sales_rank = self.pc.get_sales_rank(prod)
        if not sales_rank or sales_rank > 1500000:
          continue

        prods_buf[prod_id] = prod
        if len(prods_buf) < batch_size:
          continue

        self.process_products(prods_buf)
        prods_buf.clear()

      if prods_buf:
        self.process_products(prods_buf)
    finally:
      self.output_fh.close()

  def process_products(self, prods):
    asins = list(prods.keys())
    offers = dict()
    src_offers = self.offer_service.get_offers(self.marketplace, asins, self.condition)
    if not src_offers:
      return

    for asin, product_info in prods.items():
      prod = self.pc.convert(product_info)
      src_offer = src_offers.get(asin, None)
      offer = self.price_calc.calc_offer(src_offer)
      if not offer:
        continue

      if offer['quantity'] <= 0 or offer['price'] <= 0:
        continue

      prod['price'] = offer['price']
      prod['quantity'] = offer['quantity']

      if self.location_id:
        prod['location_id'] = self.location_id

      self.output_fh.write("{}\n".format(json.dumps(prod)))

==> em_tasks-0.1.42-py2.py3-none-any.whl: em_tasks/utils/amz_uploadable_products_filter.py <==
# -*- coding: utf-8 -*-

from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.store.store_util import StoreUtil
from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.store.models import *


class AmzUploadableProductsFilter():
  def __init__(self, marketplace, product_service, offer_service, price_caculator, output_path, location_id=None):
    self.product_service = product_service
    self.offer_service = offer_service
    self.price_calc = price_caculator
    self.location_id = location_id
    self.marketplace = marketplace.lower()
    self.pc = AmzProductConverter(self.marketplace)
    self.indice_name = 'amz_products_api_{}'.format(marketplace.lower())
    self.condition = 'new'
    self.output_path = os.path.abspath(os.path.expanduser(output_path))
    output_dir = os.path.dirname(self.output_path)
    if not os.path.isdir(output_dir):
      os.makedirs(output_dir)
    self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')
    self.sales_rank_min = 1500000
    if self.marketplace == 'us':
      self.sales_rank_min = 500000

    self.weight_max = 2.5
    if self.marketplace == 'us':
      self.weight_max = 1.5

  def filter(self):
    try:
      blacklisted_path = './amz_{}_blacklisted.txt'.format(self.marketplace)
      records = dict()
      stats = dict()
      for keyword in BlacklistKeyword.select().order_by(BlacklistKeyword.id.desc()).dicts():
        k = keyword['keywords']
        stats[k] = 0

        query = {
          'query': {
            'query': {
              'multi_match': {
                'query': k,
                'type': 'phrase',
                'fields': ['brand^10', 'title']
              }
            }
          }
        }
        for prod_id, _ in self.product_service.load_products(self.indice_name, query):
          records[prod_id] = None
          stats[k] += 1

      total_products_cnt = 0
      with open(blacklisted_path, 'w+', encoding='utf-8', errors='ignore') as fh:
        fh.write("\n".join(records.keys()))

      for k, products_cnt in stats.items():
        if not products_cnt:
          continue

        logger.debug('[BlacklistedProducts] Keyword: %s, ProductCnt: %s', k, products_cnt)

        total_products_cnt += products_cnt

      logger.debug('[BlacklistedProductsLoaded] ProductsCnt: %s', total_products_cnt)

      uploaded_prods = dict()
      for prod in StoreUtil.get_amz_products(self.marketplace):
        uploaded_prods[prod['asin']] = None

      logger.debug('[UploadedProductsLoaded] ProductsCnt: %s', len(uploaded_prods))

      prods_buf = dict()
      batch_size = 500
      for prod_id, prod in self.product_service.load_products(self.indice_name):
        exist = prod.get('existence', True)
        if not exist:
          continue

        has_only_default_variant = prod.get('has_only_default_variant', True)
        if not has_only_default_variant:
          continue

        if prod_id in records or prod_id in uploaded_prods:
          continue

        weight = self.pc.get_weight(prod)
        if weight and weight > self.weight_max:
          continue

        is_fragile = self.pc.is_fragile(prod)
        if is_fragile:
          continue

        sales_rank = self.pc.get_sales_rank(prod)
        if not sales_rank or sales_rank > self.sales_rank_min:
          continue

        prods_buf[prod_id] = prod
        if len(prods_buf) < batch_size:
          continue

        self.process_products(prods_buf)
        prods_buf.clear()

      if prods_buf:
        self.process_products(prods_buf)
    finally:
      self.output_fh.close()

  def process_products(self, prods):
    asins = list(prods.keys())
    offers = dict()
    src_offers = self.offer_service.get_offers(self.marketplace, asins, self.condition)
    if not src_offers:
      return

    for asin, product_info in prods.items():
      prod = self.pc.convert(product_info)
      src_offer = src_offers.get(asin, None)
      offer = self.price_calc.calc_offer(src_offer)
      if not offer:
        continue

      if offer['quantity'] <= 0 or offer['price'] <= 0:
        continue

      prod['price'] = offer['price']
      prod['quantity'] = offer['quantity']

      if self.location_id:
        prod['location_id'] = self.location_id

      self.output_fh.write("{}\n".format(json.dumps(prod)))
